local addonName = ...
local Master = LibStub('AceAddon-3.0'):GetAddon(GetAddOnDependencies(addonName))
local Addon = Master:CreateClass('Frame', Master.Frame)
Addon.name = addonName
_G[addonName] = Addon

Addon.master = Master
Addon.lib = LibStub('LibSharedMedia-3.0', true)

Addon.frames = {}

function Addon:New(name)
	local name = string.lower(name)
	local f = self.proto.New(self, name)
	f:Reload()
	--f:SetParent(self.header)
	Addon.frames[name] = Addon.frames[name] or f
	return f
end


function Addon:GetDefaults()
	local def
	if self and self.id and self.defaults[self.id] then 
		def = Addon.Copy(Addon.defaults[self.id])
	else
		def = Addon.Copy(Addon.defaults["focus"])
		def.x = 0
		def.y = 0
		def.point = "Center"
	end
	return def
end

function Addon:Reload()
	if self.Load then
		self:Load()
	end
	self:SetParent(self.parent or self:GetParent())
	self:Layout()
end

function Addon:Unload()
	local p = Addon.Copy(self:GetDefaults(), Addon.master.db.profile.frames[self.id])
	Addon.master.db.profile.frames[self.id] = p
	self.sets = p
	self:Layout()
	self:Rescale()
	self:Reposition()
	self:Reanchor()

end

function Addon:Disable()
	self:Free()
end


function Addon:Delete()
	Addon.master.db.profile.frames[self.id] = nil
	self:Free()
end

function Addon:Layout()
	if self.UpdateWidgets then
		self:UpdateWidgets()
	else
		local pad = self:GetPadding()
		local w, h = self.sets.width + pad, self.sets.height + pad
		self:SetSize(w, h)
		self.sets.frameStrata = self.sets.frameStrata or 4
		self.sets.frameLayer = self.sets.frameLayer or 4
		local lay = Addon.layers[self.sets.frameStrata]
		self:SetFrameStrata(lay)
		self:SetFrameLevel(self.sets.frameLayer)
	end
end


local mod = Addon.master:NewModule('Frame')

local enabledFrames = {}

function mod:OnInitialize()
	Addon.master.db.profile.units = Addon.master.db.profile.units or defaults
	Addon.unitLayouts = nil
	Addon.unitLayouts = {Default = Addon:GetDefaults()}
end

function mod:Load()
	wipe(enabledFrames)
	for unit, b in pairs(Addon.getSets().frames) do
		unit = string.lower(unit)
		local frame = Addon:New(unit)
		Addon.unitLayouts[unit] = frame.sets
		tinsert(enabledFrames, frame)
	end	

	for frameName, b in pairs(Addon.getSets().hide) do
		if frameName and _G[frameName] then
			_G[frameName]:SetParent(Addon.HIDER)
		end
	end
end

function mod:Unload()
	if enabledFrames then
		for i, frame in pairs(enabledFrames) do
			Addon.master.Frame:ForFrame(frame.id, 'Free')
			frame:Unload()
		end
	end
end

local unitFrame = {}

Addon.frames = {}

function unitFrame:New(name)
	local name = string.lower(name)
	local f = self.proto.New(self, name)
	f:Reload()
	--f:SetParent(self.header)
	Addon.frames[name] = Addon.frames[name] or f
	return f
end

local function check(source, target)
	--you may now add new defaults at will. ~Goranaws
	if not target then 
		target = {}
	end
	if not source then
		--return
	end
	for key, value in pairs(source) do
		if type(value) == 'table' then
			target[key] =check(value, target[key])
		else
			if (type(value) == 'boolean') then
				if target[key] == nil then
	target[key] = value
				end
 			else
 				target[key] = target[key] or value
 			end
		end
	end
	return target
end
Addon.check = check

function unitFrame:GetDefaults()
	local def
	if self and self.id and Addon.defaults[self.id] then 
		def = check(Addon.defaults[self.id])
	else
		def = CopyTable({
		scale = 1,
		showInOverrideUI = false,
		showInPetBattleUI = false,
		point = 'Center',
		padW = 0,
		width = 232,
		y = 0,
		x = 0,
		padH = 0,
		height = 100,
	})
	end
	return def
end

local function Copy(source, destination, looped)
	if not destination then
		destination = {}
	end
	for i, b in pairs(CopyTable(source)) do
		if (type(b) == 'table') then
			destination[i] = Copy(b, destination[i], true)
		else
				
				destination[i] = b
		end
	end
	return destination
end


function unitFrame:Reload()
	self.sets = self.sets or CopyTable(self:GetDefaults()) --check(self:GetDefaults(), self.sets)
	if self.Load then
		self:Load()
	end
	self:SetParent(self.parent or self:GetParent())
	self:Layout()
end

function unitFrame:Unload()
	local p = Copy(self:GetDefaults(), Addon.master.db.profile.frames[self.id])
	Addon.master.db.profile.frames[self.id] = p
	self.sets = p
	self:Layout()
	self:Rescale()
	self:Reposition()
	self:Reanchor()

end

function unitFrame:Disable()
	self:Free()
end

function unitFrame:Delete()
	Addon.master.db.profile.frames[self.id] = nil
	self:Free()
end

function unitFrame:Layout()
	if self.UpdateWidgets then
		self:UpdateWidgets()
	else
		local pad = self:GetPadding()
		local w, h = self.sets.width + pad, self.sets.height + pad
		self:SetSize(w, h)
		self.sets.frameStrata = self.sets.frameStrata or 4
		self.sets.frameLayer = self.sets.frameLayer or 4
		local lay = Addon.layers[self.sets.frameStrata]
		self:SetFrameStrata(lay)
		self:SetFrameLevel(self.sets.frameLayer)
	end
end

Addon.unitFrame = unitFrame
