local Addon = _G[...]
local modName = ...

--Background Handler
local widget = Addon:NewWidget('background', 'Frame')
widget.Events = {
	["UNIT_FACTION"] = 1,
}

function widget:New(parent)
	local name = ('%s_%s_BackgroundHandler'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local button = self:Bind(CreateFrame('Frame', name, parent.box))
	if button then
		button.owner = parent
	end
	return button
end

widget.defaults = {
	textureName = 'Blizzard Tooltip',
	texturePath = 'Interface\\TargetingFrame\\UI-StatusBar',
	bgPad = 0,
	edgecolor = {
		a = 100,
		r = 100,
		g = 100,
		b = 100,
	},
	edgeFile = 'Interface\\ACHIEVEMENTFRAME\\UI-Achievement-WoodBorder',
	inset = 0,
	edgeSize = 13,
	scale = 100,
	anchor = 'TopRight',
	x = -111,
	y = -26,
	width = 51,
	height = 41,
	color = {
        r = 0,
        g = 0,
        b = 0,
        a = .5, 
	},
	enable = true,
	ReactionColor = true,
}

function widget:Load()
	self:SetAllPoints(self.owner)
	self:SetFrameLevel(1)
	self:Show()
	self.noMouse = true
end

function widget:Layout()
	self:SetBackdrop({
		bgFile = self:GetTexture(),
		insets = {left = -self.sets.inset, right = -self.sets.inset, top = -self.sets.inset, bottom = -self.sets.inset},
		tile = false,
	})
	self:SetBackdropColor(self.sets.color.r, self.sets.color.g, self.sets.color.b, self.sets.color.a)
	if self.sets.enable == true then
		self:Show(1)
	else
		self:Hide(0)
	end
	local sets = self.sets
	self:Reposition()
	self:Rescale()
	local w, h = self:GetParent():GetSize()
	local width, height = w*(sets.width/100), h*(sets.height/100)
	self:SetSize(width, height)
	self:ClearAllPoints()
	self:SetPoint(sets.anchor, self:GetParent():GetParent(), sets.x, sets.y)
end

function widget:Rescale()
	self:SetScale(self.sets.scale/100)
end

function widget:Reposition()
	self:ClearAllPoints()
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	self:SetPoint(self.sets.anchor, self:GetParent(), x, y)
end

function widget:GetTexture()
	local r = Addon.lib and Addon.lib:Fetch('statusbar', self.sets.textureName)
	return r or "Interface\\TargetingFrame\\UI-StatusBar"
end

function widget:Update()

	if (self.owner.id == "player") or (not self.sets.ReactionColor) then
        self:SetBackdropColor(self.sets.color.r, self.sets.color.g, self.sets.color.b, self.sets.color.a)
    elseif ( not UnitPlayerControlled(self.owner.id)) then -- and UnitIsTapped(self.owner.id) and not UnitIsTappedByPlayer(self.owner.id) and not UnitIsTappedByAllThreatList(self.owner.id) ) then
        self:SetBackdropColor(0.5, 0.5, 0.5)
    else
        self:SetBackdropColor(UnitSelectionColor(self.owner.id))
    end
end

widget.Options = {
                    {
                        name = 'Reaction Color',
                        kind = 'CheckButton',
                        key = 'ReactionColor',
						panel = "basic",
                    },
                    {
                        name = 'Color',
                        kind = 'ColorPicker',
                        key = 'color',
                        handler = 'background',
						panel = "basic",
                    },
                    {
                        name = 'Texture',
                        kind = 'Media',
                        mediaType = 'StatusBar',
                        key = 'textureName',
                        handler = 'background',
						panel = "basic",
                    },


                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 5,
                        max = 250,
						panel = "advanced",
                    },
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 5,
                        max = 250,
						panel = "advanced",
                    },
                    {
                        name = 'Padding',
                        kind = 'Slider',
                        key = 'inset',
                        min = -30,
                        max = 30,
                        step = 1,
						panel = "advanced",
                    },
        }



