local Addon = _G[...]
local modName = ...

--Casting Bar
local widget = Addon:NewWidget('cast', 'StatusBar')
widget.Events = {
	["UNIT_SPELLCAST_CHANNEL_START"] = 1,
	["UNIT_SPELLCAST_CHANNEL_STOP"] = 1,
	["UNIT_SPELLCAST_CHANNEL_UPDATE"] = 1,
	["UNIT_SPELLCAST_DELAYED"] = 1,
	["UNIT_SPELLCAST_FAILED"] = 1,
	["UNIT_SPELLCAST_FAILED_QUIET"] = 1,
	["UNIT_SPELLCAST_INTERRUPTED"] = 1,
	["UNIT_SPELLCAST_INTERRUPTIBLE"] = 1,
	["UNIT_SPELLCAST_NOT_INTERRUPTIBLE"] = 1,
	["UNIT_SPELLCAST_SENT"] = 1,
	["UNIT_SPELLCAST_START"] = 1,
	["UNIT_SPELLCAST_STOP"] = 1,
	["UNIT_SPELLCAST_SUCCEEDED"] = 1,
}


function widget:New(parent)
	local name = ('%s_%s_CastingBar'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local opac = CreateFrame('Frame', nil, parent.box)--opacity control
	local bar = self:Bind(CreateFrame('StatusBar', name, opac, 'DU_CastingBarFrameTemplate'))
	bar.owner = parent
	bar.opac = opac
	bar:Hide()
	bar:SetAttribute('unit', parent.id)
	CastingBarFrame_SetLook(bar, 'UNITFRAME')
	bar.Text:SetHeight(12)
	bar.id = parent.id
	bar.time = 	bar.time or 	bar:CreateFontString(nil, 'OVERLAY', 'TextStatusBarText')
	bar.time:SetTextColor(1.0,1.0,1.0)
	bar.time:SetAllPoints(bar)
	bar.drop = 	bar.drop or CreateFrame('StatusBar', nil, bar)
	bar.drop:SetMinMaxValues(0,1)
	bar.drop:SetValue(1)
	bar.drop:SetAllPoints(true)
	bar.drop:SetFrameLevel(bar:GetFrameLevel()-2)
	
	bar.testTexture = 	bar.testTexture or CreateFrame('StatusBar', nil)
	bar.testTexture:SetMinMaxValues(0,1)
	bar.testTexture:SetValue(1)
	bar.testTexture:SetAllPoints(bar)
	bar.testTexture:SetFrameLevel(bar:GetFrameLevel()-2)
	
	bar.spark = bar.Spark
	bar.Spark = bar:CreateTexture(nil, 'ARTWORK')
	bar.Spark:SetPoint("Center")
	bar.Spark :SetSize(1,1)
	bar.Border:SetParent(Addon.HIDER)
	bar.BorderShield:SetParent(Addon.HIDER)
	bar.Icon:SetParent(Addon.HIDER)
	bar.Flash:SetParent(Addon.HIDER)
	return bar
end

widget.defaults = {
	texturePath = 'Interface\\TargetingFrame\\UI-StatusBar',
	textureName = 'Blizzard',
	frameLevel = 2,
	frameLayer = 4,
	scale = 100,
	alpha = 100,
	width = 117,
	y = 17,
	x = 7,
	anchor = 'Left',
	height = 16,
	enable = true,
	showBG = false,
	mouseOverTooltip = true,
	arc = 0,
    textX = 0,
    textY = 0,
	textPoint = "Left",
	maxTextWidth = 98,
	sparkY = 0,
	Sparkarc = 0,
	multi = true,
    rotate = nil,
	orient = "HORIZONTAL",
	backdropOpacity = 50,
	timePoint = "Right",
	timeX = 0,
	timeX = 0,
}

function widget:Load()
	CastingBarFrame_OnLoad (self, self.id)
	self:SetScript('OnShow', self.OnShow)
	self:SetScript('OnEvent', nil)
	self:Show()
end

local _hori = {
	'LEFT',
	'CENTER',
	'RIGHT',
}

local _vert = {
	'TOP',
	'MIDDLE',
	'BOTTOM',
}

function widget:SetSpark()
    if self.sets.rotate == true then
        self.spark:SetRotation(math.rad(-90))
    else
        self.spark:SetRotation(math.rad(0))
    end
end

function widget:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	self:SetPoint(sets.anchor, self:GetParent():GetParent(), sets.x, sets.y)
	local width, height = (sets.width), (sets.height)
	self:SetSize(width, height)
	if sets.scale < 25 then
		sets.scale = sets.scale *100
	end
	self:SetScale(sets.scale/100)
	self.opac:SetAlpha(sets.alpha/100)
	local anchor = sets.anchor

	sets.justifyH=sets.justifyH or 2
	sets.justifyV = sets.justifyV or 2
	self.Text:SetJustifyH(_hori[sets.justifyH] or sets.justifyH or "CENTER") -- Sets horizontal text justification ('LEFT','RIGHT', or 'CENTER')
	self.Text:SetJustifyV(_vert[sets.justifyV] or sets.justifyV or 'MIDDLE') -- Sets vertical text justification ('TOP','BOTTOM', or 'MIDDLE')
	

	
	self.Text:SetWidth(sets.maxTextWidth)

	local lay = Addon.layers[sets.frameLayer]
	self:SetFrameStrata(lay)
	self:SetFrameLevel(sets.frameLevel)
    if sets.showBG then
        self.drop:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
        self.drop:SetStatusBarColor(0,0,0,self.sets.backdropOpacity/100)
    else
        self.drop:SetStatusBarTexture("")
    end
	
	
	if sets.testTexture then
        self.testTexture:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
        self.testTexture:SetStatusBarColor(1,0,0,self.sets.backdropOpacity/100)
    else
        self.testTexture:SetStatusBarTexture("")
    end
	
	
	self:SetOrientation(sets.orient) -- "HORIZONTAL" or "VERTICAL"
	self.drop:SetOrientation(sets.orient)
	self.testTexture:SetOrientation(sets.orient)
	
    if (self.prevTexture ~= self:GetMediaPath('statusbar', self.sets.textureName)) or
    (self.prevTexture ~= self.sets.texturePath) then
	    self:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
    end
    self:SetRotatesTexture(self.sets.rotate)
    self.drop:SetRotatesTexture(self.sets.rotate)
    self.testTexture:SetRotatesTexture(self.sets.rotate)
	
	if self.time then
		local _time = self.time
		if self.sets.timeDisable then
			_time:Hide()
		else
			_time:ClearAllPoints()
			_time:SetPoint(self.sets.timePoint, self.sets.timeX, self.sets.timeY)
		end
	end
	
    self:SetSpark()
end

function widget:GetMediaPath(kind, fileName)
	if Addon.lib then
		self.sets.texturePath = Addon.lib and Addon.lib:Fetch(kind, fileName)
	end
	return (Addon.lib and Addon.lib:Fetch(kind, fileName))
end

local function SettingsChangeDetected(self, sets)
    local changeDetected
    for setting, value in pairs(sets) do
        if self["previous-"..setting] ~= value then
            self["previous-"..setting] = value
            changeDetected = true
        end
    end
    if changeDetected then
        return true
    end    
end

local function nochange(self, kind, name, subText, text, texture, startTime, endTime, castID)
    if self.prevkind ~= kind or 
    self.prevname ~= name or
    self.prevsubText ~= subText or
    self.prevtext ~= text or
    self.prevtexture ~= texture or
    self.prevstartTime ~= startTime or
    self.prevendTime ~= endTime or
    self.prevcastID ~= castID
    or SettingsChangeDetected(self, self.sets) then
        self.prevkind = kind
        self.prevname = name
        self.prevsubText = subText
        self.prevtext = text
        self.prevtexture = texture
        self.prevstartTime = startTime
        self.prevendTime = endTime
        self.prevcastID = castID
    else
        return
    end
end

function widget:OnEvent(event, name, castID, spellID)
	return CastingBarFrame_OnEvent (self, event, self.id, name, nil, castID or self.castID, spellID or select( 7, GetSpellInfo(name)))
end

function widget:Update(elapsed)
	if self.noUpdate then
		return
	end
	local curGUID = UnitGUID(self.id)
	if (curGUID ~= self.lastGUID) or not curGUID then
		if self.lastGUID then
			self:GetScript('OnShow')(self)
		end
		if not curGUID then
			return
		end
	end
	local kind, name, subText, text, texture, startTime, endTime, _,  castID = self:isCasting()
    
    if nochange(self, kind, name, subText, text, texture, startTime, endTime, castID) then
        return
    end
	self.seconds = self.seconds or ''
	if kind then
		--Events to mimmick
		--cast start
		--cast delayed
		--channel Start
		if kind == 'cast' then
			if not self.startTime then
				self:OnEvent('UNIT_SPELLCAST_START', name, castID)
			else
				if self.endTime ~= endTime then
					self:OnEvent('UNIT_SPELLCAST_DELAYED', self.spellname, self.castID, self.spellID)
				end
			end
		elseif kind == 'channel' then
					self:OnEvent('UNIT_SPELLCAST_CHANNEL_START', name, self.castID)
		end
		self.begin = self.begin or GetTime()
		self.kind = kind
		self.spellID = select( 7, GetSpellInfo(name))
		self.startTime = startTime
		self.endTime = endTime
		self.spellname = name
		if kind == 'cast' then
			self.castID = castID
		else
			self.castID = castID or 0
		end
		self.stop = nil
	elseif self.kind then
		local time = GetTime()
		if ((time) < ((self.endTime/1000) - .5)) then
			if self.kind == 'cast' then
					self:OnEvent('UNIT_SPELLCAST_INTERRUPTED', self.spellname, self.castID, self.spellID, self.spellID)
				self.casting = false
                self.spark:Hide()
			elseif self.kind == 'channel' then
					self:OnEvent('UNIT_SPELLCAST_CHANNEL_STOP', self.spellname, self.castID, self.spellID, self.spellID)
				self.channeling = false
                self.spark:Hide()
			end
		else
			if self.kind == 'cast' then
				self:OnEvent('UNIT_SPELLCAST_STOP', self.spellname, self.castID, self.spellID, self.spellID)
				self.casting = false
                self.spark:Hide()
			elseif self.kind == 'channel' then
				self:OnEvent('UNIT_SPELLCAST_CHANNEL_STOP', self.spellname, self.castID, self.spellID, self.spellID)
				self.channeling = false
                self.spark:Hide()
			end
		end
		self.kind = nil
		self.spellID = nil
		self.startTime = nil
		self.endTime = true
		self.spellname = nil
		self.begin = nil
		self.seconds = ''
		self.stop = true
	end
	if name then
        if self.stop then
            self.time:SetText('')
            self.Text:SetText('')
        else
            if self.endTime then
                self.seconds = string.format('%.1f' ,(self.endTime / 1000) - GetTime())
                self.time:SetText(self.seconds)
            end
        end
    end
	if name then
        self.Text:Show()
        self.Text:ClearAllPoints()
		self.Text:SetAlpha(1)
		self.Text:SetPoint(self.sets.textPoint, self, self.sets.textX, self.sets.textY)
        self.Text:SetText(name)
        self.spark:Show()
	end
    if self.value then
        local Sx, Sy = self:GetCenter()
		Sx, Sy = Sx or 0, Sy or 0
        local width, height = self:GetSize()
        if self.rotated ~= self.sets.rotate then
            self.spark:ClearAllPoints()
        end
        local sparkPosition
        local y
        if ( self.spark ) and (self.value and self.maxValue) then
            if self.sets.orient == "HORIZONTAL" then
                sparkPosition = (self.value / self.maxValue) * width;
                y = ((((height/2)/((width/2)^2) )*(((Sx - (Sx + (sparkPosition - (width/2)))))^2)) * (self.sets.Sparkarc/100)) + self.sets.sparkY
			    self.spark:SetPoint("CENTER", self, "LEFT", sparkPosition, y or self.spark.offsetY or 2);
            else
                sparkPosition = (self.value / self.maxValue) * height;
                y = ((((width/2)/((height/2)^2) )*(((Sy - (Sy + (sparkPosition - (height/2)))))^2)) * (self.sets.Sparkarc/100)) + self.sets.sparkY
			    self.spark:SetPoint("CENTER", self, "BOTTOM", y or self.spark.offsetY or 2, sparkPosition );
            end
            self.spark.offsetY = y
		end
    end
	
	if self.sets.testTexture then
        self.Text:SetText(self, "This Is a Test! You have been tested!")
	end
	
	self.lastGUID = curGUID
end

function widget:isCasting()
	if UnitCastingInfo(self.id) then
		return 'cast', UnitCastingInfo(self.id)
	elseif UnitChannelInfo(self.id) then
		return 'channel', UnitChannelInfo(self.id)
	end
end

function widget:OnShow()
	local kind,_, _, _, _, startTime, endTime = self:isCasting()
	if (kind == 'cast') then
		if ( startTime ) then
			self.value = (GetTime() - (startTime / 1000))
		end
	elseif (kind == 'channel') then
		if ( endTime ) then
			self.value = ((endTime / 1000) - GetTime())
		end
	else
		self.fadeOut = 0
		self.holdTime = 0
		self:Hide()
	end
end

function widget:UpdateValue(key, value)
	self.sets[key] = value
	self:Layout()
end

function widget:GetValue(key)
	return self.sets[key]
end

widget.Options = {				
                {
                    name = 'Test Mode',
                    kind = 'CheckButton',
                    key = 'testTexture',
					panel = "basic",
                },

                {
                    name = 'Texture',
                    kind = 'Media',
                    key = 'textureName',
                    mediaType = 'StatusBar',
					panel = "texture",
                },
                {
                    name = 'Rotate 90',
                    kind = 'CheckButton',
                    key = 'rotate',
					panel = "texture",
                },
                {
                    name = 'Growth',
                    kind = 'Menu',
                    key = 'orient',
                    table = {"HORIZONTAL", "VERTICAL"},
					panel = "texture",
                },
                {
                    name = 'Background',
                    kind = 'CheckButton',
                    key = 'showBG',
					panel = "texture",
                },
                {
                    name = 'Background Alpha',
                    kind = 'Slider',
                    key = 'backdropOpacity',
                    min = 0,
                    max = 100,
					panel = "texture",
                },
                {
                    name = 'Width',
                    kind = 'Slider',
                    key = 'maxTextWidth',
                    min = 1,
                    max = 500,
					panel = "text",
                },


                {
                    name = 'Offset',
                    kind = 'Slider',
                    key = 'sparkY',
                    min = -150,
                    max = 150,
					panel = "spark",
                },
                {
                    name = 'Arc',
                    kind = 'Slider',
                    key = 'Sparkarc',
                    min = -300,
                    max = 300,
					panel = "spark",
                },

                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 10,
                        max = 400,
						panel = "Basic",
                    }, 
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 10,
                        max = 200,
						panel = "basic",
                    },
                    {
                        name = 'Tooltip',
                        kind = 'CheckButton',
                        key = 'mouseOverTooltip',
						panel = "advanced",
                    },
                    {
                        name = 'Disable',
                        kind = 'CheckButton',
                        key = 'timeDisable',
						panel = "Time",
                    },
					{
                        name = 'Vertical Offset',
                        kind = 'Slider',
                        key = 'timeY',
                        min = -200,
                        max = 200,
						panel = "time",
                    },
					{
                        name = 'Horizontal Offset',
                        kind = 'Slider',
                        key = 'timeX',
                        min = -200,
                        max = 200,
						panel = "time",
                    },
					
					{
						name = 'Position',
						kind = 'Menu',
						key = 'timePoint',
						table = Addon.anchors,
						panel = "time",
					},
					{
                        name = 'Vertical Offset',
                        kind = 'Slider',
                        key = 'textY',
                        min = -200,
                        max = 200,
						panel = "text",
                    },
					{
                        name = 'Horizontal Offset',
                        kind = 'Slider',
                        key = 'textX',
                        min = -200,
                        max = 200,
						panel = "text",
                    },
					
					{
						name = 'Position',
						kind = 'Menu',
						key = 'textPoint',
						table = Addon.anchors,
						panel = "text",
					},
					
					{
                        name = 'Justify Vert',
                        kind = 'Slider',
                        key = 'justifyV',
                        min = 1,
                        max = 3,
						panel = "text",
                    },
                    {
                        name = 'Justify Hori',
                        kind = 'Slider',
                        key = 'justifyH',
                        min = 1,
                        max = 3,
						panel = "text",
                    },
        }
