local Addon = _G[...]
local modName = ...

--Debuff Bar
local widget = Addon:NewWidget('debuffs', 'Frame')
widget.Events = {
	["UNIT_AURA"] = 1,
}

function widget:New(parent)
	local name = ('%s_%s_DebuffBar'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local bar = self:Bind(CreateFrame('Frame', name, parent.box))
	bar.owner = parent
	bar.drop = bar.drop or CreateFrame('Frame', nil, bar)
--	bar.drop:SetFrameLevel(bar:GetFrameLevel()-1)
	bar.drop:SetAllPoints(bar)
	bar.filter = 'HARMFUL'
	bar.kind = 'debuff'
	return bar
end

widget.defaults = {
	columns = 6,
	scale = 76,
	rows = 3,
	alpha = 100,
	y = -73,
	x = 0,
	isLeftToRight = true,
	spacing = 0,
	anchor = 'Top',
	enable = true,
	showBG = false,
	isTopToBottom = true,
	color = {
        a = 1,
        r = 0,
        g = 0,
        b = 0,
	},
	sort = 'Duration',
}

function widget:Load()
	self:SetAttribute('unit', self.owner.id)
	self:SetAttribute('filter', self.filter)
	self:EnableMouse(false)
	self.noMouse = true
	self.id = self.owner.id
	self.icons = self.icons or {}
	self.total = self.total or 0
end

function widget:Layout()
	local sets = self.sets
	if sets.enable ~=true then
		self:Hide()
		self.noUpdate = true
		return
	else
		--self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	local x = sets.x/(sets.scale/100)
	local y = sets.y/(sets.scale/100)
	self:SetPoint(sets.anchor, self:GetParent(), x, y)
	if sets.scale < 25 then
		sets.scale = sets.scale *100
	end
	self:SetScale(sets.scale/100)
	self:SetAlpha(sets.alpha/100)
	for i = 1, sets.rows * sets.columns do
		self:GetOrCreateIcon(i)
	end
	local w, h = self.icons[1]:GetSize()
	w, h =  w + sets.spacing,  h + sets.spacing
	local newWidth =  w * sets.columns - sets.spacing
	local newHeight = h * sets.rows    - sets.spacing
	self:SetSize(newWidth, newHeight)
	local cols, rows = sets.columns, sets.rows
	local isLeftToRight = sets.isLeftToRight
	local isTopToBottom = sets.isTopToBottom
	for i, icon in pairs(self.icons) do
		local col, row = (cols-1) - (i-1) % cols, rows - ceil(i / cols)
		if isLeftToRight then
			col = (i-1) % cols
		end
		if isTopToBottom then
			row = ceil(i / cols) - 1
		end
		icon:ClearAllPoints()
		icon:SetPoint('TOPLEFT', w*col, -(h*row))
		icon:Show()
	end
	if sets.showBG then
		self.drop:SetBackdrop(self.bg)
		self.drop:SetBackdropColor(self.sets.color.r, self.sets.color.g, self.sets.color.b, self.sets.color.a)
	else
		self.drop:SetBackdrop(nil)
	end
end

function widget:Update(elapsed)
	if self.sets.enable ~= true then
		return
	end
	self:GetAuras()
	self:DisplayOrder()
	self:UpdateTooltip()
end

function widget:GetAuras()
	local numDisplayed = self.sets.columns * self.sets.rows
	self.auras = self.auras or {}
	wipe(self.auras)
	local i = 1
	while i do
		local name, _, icon, count, debuffType, remaining, expiration = UnitAura(self.id, i, self.filter)
		if name then
			tinsert(self.auras, {name, icon, count, expiration, remaining, i,})
			i = i + 1
		else
			i = nil
		end
	end
end

function widget:DisplayOrder()
	for i, icon in pairs(self.icons) do --clear any unused icons.
		if i > #self.auras then
			self:ClearIcon(icon)
		end
	end
	if self.sets.sort == 'Duration' then
		table.sort(self.auras,	function(a, b)
			local A, B = a[5], b[5]
			if A == 0 then--durations of 0 are considered 'infinite' for sorting purposes
				A = 1000000000000000000000000
			end
			if B == 0 then
				B = 1000000000000000000000000
			end
			return A < B
		end)
	elseif self.sets.sort == 'Alphabetical' then
		table.sort(self.auras,	function(a, b)
			return a[1] < b[1]
		end)
	end
	if self.sets.reverse then
		local reversedTable = {}
		local itemCount = #self.auras
		for k, v in ipairs(self.auras) do
			reversedTable[itemCount + 1 - k] = v
		end
		self.auras = reversedTable
	end
	local numDisplayed = self.sets.columns * self.sets.rows
	for i, info in pairs(self.auras) do --update displayable auras
		if i <= (numDisplayed) then
			self:SetIcon(i, unpack(info))
		end
	end
end

function widget:UpdateTooltip()
	local hasMouse
	for i, b in pairs(self.icons) do
		if self.auras[i] then
			if MouseIsOver(b) then
				hasMouse = true
				GameTooltip:SetOwner(self)
				GameTooltip:SetUnitAura(self.id, self.auras[i][6], self.filter)
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint('BottomRight', b, 'TopLeft')
				break
			end
		end
	end
	if hasMouse ~= true then
		if GameTooltip:GetOwner() == self then
			GameTooltip:Hide()
		end
	end
end

function widget:GetOrCreateIcon(i)
	local t = self.icons[i] or self:CreateTexture(self:GetName()..self.kind..i, 'ARTWORK')
	if not self.icons[i] then
		self.icons[i] = t
		t:SetSize(32, 32)
		t.cooldown = CreateFrame('Cooldown', t:GetName()..'Cooldown', self, 'CooldownFrameTemplate')
		t.cooldown:SetAllPoints(t)
		t.count = self:CreateFontString(t:GetName()..'Count', 'OVERLAY', 'GameFontNormal')
		t.count:SetPoint('BottomRight', t)
		t.isShown = nil
		t.index = i
	end
	return t
end

function widget:ClearIcons()

end

function widget:SetIcon(i, name, image, count, expiration, remaining)
	local icon = self:GetOrCreateIcon(i)
	if icon.lastImage ~= image then --only make a change, if there is a change.
		icon.lastImage = image
		icon:SetTexture(image)
		icon:SetTexCoord(.1,.9,.1,.9)
	end
	if (icon.lastExpiration ~= expiration) then
		icon.lastExpiration = expiration
		--CooldownFrame_SetTimer(icon.cooldown, expiration - remaining, remaining, 1)
	end
	if icon.lastCount ~= count then
		icon.lastCount = count
		if count == 0 then
			count = ''
		end
		icon.count:SetText(count)
	end
	icon.empty = nil
end

function widget:ClearIcon(icon)
	if not icon.empty then --don't clear if it's already clear.
		icon.empty = true
		icon.lastImage = nil
		icon:SetTexture('')
		icon.lastCount = nil
		icon.count:SetText('')
		icon.lastExpiration = nil
		icon.cooldown:Hide()
	end
end

widget.bg = {
	bgFile = 'Interface\\Tooltips\\UI-Tooltip-Background',
	insets = {left = 0, right = 0, top = 0, bottom = 0},
	tile = false,
}


widget.Options = {
                    {
                        name = 'Columns',
                        kind = 'Slider',
                        key = 'columns',
                        min = 1,
                        max = 10,
						panel = "basic",
                    },
                    {
                        name = 'Rows',
                        kind = 'Slider',
                        key = 'rows',
                        min = 1,
                        max = 10,
						panel = "basic",
                    },
                    {
                        name = 'Spacing',
                        kind = 'Slider',
                        key = 'spacing',
                        min = -13,
                        max = 30,
						panel = "basic",
                    },
                    {
                        name = 'Reverse',
                        kind = 'CheckButton',
                        key = 'reverse',
						panel = "display",
                    },
                    {
                        name = 'Top To Bottom',
                        kind = 'CheckButton',
                        key = 'isTopToBottom',
						panel = "display",
                    },
                    {
                        name = 'Left To Right',
                        kind = 'CheckButton',
                        key = 'isLeftToRight',
						panel = "display",
                    },
    
                    {
                        name = 'Sort',
                        kind = 'Menu',
                        key = 'sort',
                        table = {
                            'Duration',
                            'Alphabetical',
                            'Normal',
                        },
						panel = "display",
                    },
                    {
                        name = 'Backdrop',
                        kind = 'CheckButton',
                        key = 'showBG',
						panel = "display",
                    },
                    {
                        name = 'Backdrop Color',
                        kind = 'ColorPicker',
                        key = 'color',
						panel = "display",
                    },

        }