local Addon = _G[...]
local modName = ...

local Masque = LibStub('Masque', true)


local TEXTURE = [[Interface\Addons\]]..modName..[[\textures\Focus]]

--Focus Button
local widget = Addon:NewWidget('focus', 'Button')
widget.Events = {
	["UNIT_PORTRAIT_UPDATE"] = 1,
	["UNIT_MODEL_CHANGE"] = 1,
}

function widget:New(parent)
	local name = ('%s_%s_FocusButton'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local button = self:Bind(CreateFrame('Button', name, parent.box, 'SecureActionButtonTemplate'))
	button.owner = parent
	button.icon = button:CreateTexture(nil, 'ARTWORK')
	button.icon:SetAllPoints(button)
	
	button:SetNormalTexture("")

	
	button:SetFrameLevel(parent:GetFrameLevel()+ 10)
	button:SetSize(64, 64)
	button.icon:SetTexture(TEXTURE)
	return button
end

function widget:SetChecked()

end
function widget:GetChecked()
	return false
end

widget.defaults = {
	x = -42,
	y = -12,
	anchor = 'TopRight',
	enable = true,
	scale = 95,
	alpha = 100,
	usePortrait = true,
	mouseOverTooltip = true,
}

function widget:Load()
	self:SetAttribute('unit', self.owner.id)
	self:RegisterForClicks('AnyUp')
	self:SetAttribute('*type1', 'target')
			
	if self.owner.id ~= 'focus' then
		self:SetAttribute('*type2', 'focus')
		self.Tooltip = 'Right-Click to Set Focus.'
	else
		self:SetAttribute('*type2', 'macro')
		self:SetAttribute('macrotext', '/clearfocus')
		self.Tooltip = 'Right-Click to Clear Focus.'
	end
	if not self.masqued then
		self.masqued = true
		
			if Masque then
				local group = Masque:Group(modName, "Focus Macros")

				group:AddButton(self, { Icon = self.icon, Border = self.IconBorder})

				if group.db.disabled then
					self:ApplyDefaultTheme(self)
				end
			end
		
	end
	self:Show()
	self:EnableMouse(true)
	self:SetScript('OnEnter', self.OnEnter)
	self:SetScript('OnLeave', self.OnExit)
end

function widget:ApplyDefaultTheme()
	if sets.usePortrait then
		SetPortraitTexture(self.icon, self.owner.id)
		if (not self.icon:GetTexture()) or (self.icon:GetTexture() == "") then
			self.icon:SetTexture(TEXTURE)
		end
		self.icon:SetVertexColor(1, 1, 1)
	else
		local localizedClass, englishClass = UnitClass(self.owner.id)
		local classColor = RAID_CLASS_COLORS[englishClass]
		if classColor then
			r, g, b = classColor.r, classColor.g, classColor.b
		else
			r, g, b = 1, 1, 1
		end
		if self.icon:GetTexture() ~= TEXTURE then
			self.icon:SetTexture(TEXTURE)
		end
		self.icon:SetVertexColor(r, g, b)
	end
end

function widget:Layout()
    if InCombatLockdown() then
        return
    end
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		self:SetText('')
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	
	self:SetPoint(sets.anchor, self:GetParent(), x, y)

	if sets.scale < 25 then
		sets.scale = sets.scale *100
	end
	self:SetScale(sets.scale/100)
	self:SetAlpha(sets.alpha/100)
	if self:GetParent().unitButton then
		self:GetParent().unitButton:SetFrameStrata("BACKGROUND")
	end
end

function widget:Update()
	local sets = self.sets
	
	if sets.usePortrait then
		SetPortraitTexture(self.icon, self.owner.id)
		if (not self.icon:GetTexture()) or (self.icon:GetTexture() == "") then
			self.icon:SetTexture(TEXTURE)
		end
		self.icon:SetVertexColor(1, 1, 1)
	else
		local localizedClass, englishClass = UnitClass(self.owner.id)
		local classColor = RAID_CLASS_COLORS[englishClass]
		if classColor then
			r, g, b = classColor.r, classColor.g, classColor.b
		else
			r, g, b = 1, 1, 1
		end
		if self.icon:GetTexture() ~= TEXTURE then
			self.icon:SetTexture(TEXTURE)
		end
		self.icon:SetVertexColor(r, g, b)
	end
end

function widget:OnEnter()
	UnitFrame_OnEnter(self:GetParent())
	GameTooltip:AddLine(self.Tooltip)
	GameTooltip:AppendText('')
end

function widget:OnExit()
    if MouseIsOver(self:GetParent()) then
	    UnitFrame_OnEnter(self:GetParent())
    else
	    UnitFrame_OnLeave(self:GetParent())
    end
end

widget.Options = {
                    {
                        name = 'Portrait Icon',
                        kind = 'CheckButton',
                        key = 'usePortrait',
						panel = "basic",
                    },

                    {
                        name = 'Tooltip',
                        kind = 'CheckButton',
                        key = 'mouseOverTooltip',
						panel = "advanced",
                    },

        }
