local Addon = _G[...]
local modName = ...
--Model Frame
local widget = Addon:NewWidget('model', 'PlayerModel')

widget.Events = {
	["UNIT_PORTRAIT_UPDATE"] = 1,
	["UNIT_MODEL_CHANGED"] = 1,
}

function widget:New(parent)
	local name = ('%s_%s_ModelFrame'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local model = self:Bind(CreateFrame('PlayerModel', name, parent.box, 'ModelWithZoomTemplate'))
	model.owner = parent
	model:SetFrameLevel(1)
	model:RegisterEvent('UNIT_MODEL_CHANGED')
	return model
end

widget.defaults = {
	zoom = 0.7,
	frameLevel = 1,
	frameLayer = 1,
	scale = 1,
	justifyV = 'CENTER',
	width = 133,
	y = -45,
	x = -110,
	justifyH = 'CENTER',
	rotation = 0.449085388183594,
	anchor = 'BottomLeft',
	height = 197,
	enable = false,
	clickThru = true,
	rotation = 0,
	zoom = 0,
}

function widget:Load()
	self:RegisterForDrag(false)
	self:EnableMouse(false)
	self:SetScript('OnEvent', self.OnEvent)
	self:SetScript('OnShow', self.OnShow)
	self:SetScript('OnMouseWheel', self.OnMouseWheel)
	self:SetScript('OnMouseUp', self.OnMouseUp)
end

function widget:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	self:SetPoint(sets.anchor, self:GetParent(), sets.x, sets.y)
	local w, h = self:GetParent():GetSize()
	local width, height = sets.width, sets.height
	self:SetSize(width, height)
	self:SetScale(sets.scale/100)
	self:EnableMouse(false)
	--self:EnableMouseWheel(false)



	local rot = (self.sets.rotation/360)*6.28318306053847
	local zoo = self.sets.zoom/100

	self.zoomlevel = zoo
	self.rotation = rot

	self:SetRotation(rot or MODELFRAME_DEFAULT_ROTATION)
	self:SetPortraitZoom(zoo or MODELFRAME_MIN_ZOOM)

	local lay = Addon.layers[sets.frameLayer]
	self:SetFrameStrata(lay)
	self:SetFrameLevel(sets.frameLevel)
end

function widget:Update()
	if self.noUpdate then
		return
	end
	local unit = self.owner.id
	local dead = UnitIsGhost(unit) or UnitIsDead(unit)
	local sets = self.sets
	if (max == 0 or dead) and (self:GetAlpha() ~= .5) then
		self:SetAlpha(.5)
	elseif self:GetAlpha() ~= 1 then
		self:SetAlpha(1)
	end
	self.lastUnit = self.lastUnit or UnitGUID(unit)
	if self.lastUnit ~= UnitGUID(unit) then
		self.lastUnit = UnitGUID(unit)
		self:GetScript('OnEvent')(self)
	end
end

function widget:OnEvent(...)
	if self.owner.id and UnitName(self.owner.id) then
		self:SetUnit(self.owner.id)
	end
end

function widget:OnShow()
	self:GetScript('OnEvent')(self)
	self:SetRotation(self.rotation or MODELFRAME_DEFAULT_ROTATION)
	self:SetPortraitZoom(self.zoomlevel or MODELFRAME_MIN_ZOOM)
end

function widget:OnMouseWheel(...)
	Model_OnMouseWheel(self, ...)
	if self.zoomLevel ~= self.sets.zoom then
		self.sets.zoom = self.zoomLevel
	end
end

function widget:OnMouseUp(...)
	Model_OnMouseUp(self, ...)
	if self.rotation ~= self.sets.rotation then
		self.sets.rotation = self.rotation
	end
end

widget.Options = {

                    {
                        name = 'Rotation',
                        kind = 'Slider',
                        key = 'rotation',
                        min = 0,
                        max = 360,
						panel = "basic",
                    },	

                    {
                        name = 'Zoom',
                        kind = 'Slider',
                        key = 'zoom',
                        min = -50,
                        max = 300,
						panel = "basic",
                    },

                    {
                        name = 'Death Effect',
                        kind = 'CheckButton',
                        key = 'deathEffect',
						panel = "basic",
                    },
                    {
                        name = 'Click Thru',
                        kind = 'CheckButton',
                        key = 'clickThru',
						panel = "basic",
                    },

                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 10,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 10,
                        max = 400,
						panel = "advanced",
                    },

        }

