local Addon = _G[...]
local modName = ...

--Name Text
local widget = Addon:NewWidget('name', 'Frame')

function widget:New(parent)
	local name = ('%s_%s_NameText'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local control = self:Bind(CreateFrame('Frame', name, parent.box))
	control.owner = parent
	control:SetFrameLevel(40)
	local string = control:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	string:SetAllPoints(control)
	control.text = string
	self.key = "name"
	self.ID =  parent.id
	--RegisterNewUnitEvent(self, "UNIT_NAME", "on", UnitName)
	return control
end

widget.defaults = {
	enable = true,
	justifyV = 1,
	width = 100,
	x = -50,
	y = 19,
	justifyH = 2,
	anchor = 'Center',
	height = 12,
	scale = 100,
}

local _hori = {
	'LEFT',
	'CENTER',
	'RIGHT',
}

local _vert = {
	'TOP',
	'MIDDLE',
	'BOTTOM',
}

function widget:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		self.text:SetText('')
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:SetSize(sets.width, sets.height)
	self.text:SetJustifyH(_hori[sets.justifyH]) -- Sets horizontal text justification ('LEFT','RIGHT', or 'CENTER')
	self.text:SetJustifyV(_vert[sets.justifyV]) -- Sets vertical text justification ('TOP','BOTTOM', or 'MIDDLE')
	self:SetScale(self.sets.scale/100)
	self:ClearAllPoints()
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	self:SetPoint(self.sets.anchor, self:GetParent(), x, y)
end

function widget:Update()
	if self.noUpdate then
		return
	end
	local unit = self.owner.id
	self.text:SetText(UnitName(unit))
end

widget.Options = {
                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 10,
                        max = 400,
                        step = 1,
						panel = "basic",
                    },
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 12,
                        max = 240,
                        step = 14,
						panel = "basic",
                    },

                    {
                        name = 'Justify Vert',
                        kind = 'Slider',
                        key = 'justifyV',
                        min = 1,
                        max = 3,
						panel = "advanced",
                    },
                    {
                        name = 'Justify Hori',
                        kind = 'Slider',
                        key = 'justifyH',
                        min = 1,
                        max = 3,
						panel = "advanced",
                    },


        }
		