local Addon = _G[...]
local modName = ...

local widget = Addon:NewWidget('overlay', 'Frame')
widget.Events = {
	UNIT_CLASSIFICATION_CHANGED = 1,
}

function widget:New(parent)
	local name = ('%s_%s_overlay'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local bar = self:Bind(CreateFrame('Frame', name, parent.box))
	bar:EnableMouse(false)
	--self:EnableMouse(false)

	bar.texture = bar:CreateTexture(nil, 'ARTWORK')
	bar.texture:SetAllPoints(bar)
	bar.owner = parent
	bar.glow = bar:CreateTexture(nil, 'ARTWORK')
	bar.glow:SetPoint('TopLeft', 35, -8)
	bar.glow:SetSize(190, 66)
	bar.glow:SetTexture('Interface\\CharacterFrame\\UI-Player-Status')
	bar.glow:SetTexCoord(0, 0.74609375, 0, 0.53125)
	bar.glow:SetBlendMode('ADD')
	parent.threatIndicator = bar.glow
	parent.unit = parent.id
	bar.glow.unit = parent.id
	bar.statusCounter = 0
	bar.statusSign = -1
	bar:SetAllPoints(self)
	return bar
end

widget.defaults = {
	enable = true,
	anchor = 'Center',
	x = 0,
	y = 0,
	flipL2R = false
}

function widget:Load()
	self:EnableMouse(false)
	self.noMouse = true
	self.id = self.owner.id
end

function widget:Layout()
	local sets = self.sets
	if sets.enable ~=true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end

	local w, h = self.owner.sets.width, self.owner.sets.height
   
    self:SetSize(w, h)
	self:EnableMouse(false)
	self:ClearAllPoints()
	self:SetAllPoints(self:GetParent())
			if self.sets.flipL2R == true then
			self.texture:SetTexCoord(1, 0.09375, 0, 0.78125)
		else
			self.texture:SetTexCoord(0.09375, 1.0, 0, 0.78125)
		end
end

function widget:Update(elapsed)
	if (self.sets.enable ~= true) or self.noUpdate then
		return
	end

    if ( UnitHasVehiclePlayerFrameUI(self.owner.id) ) then
		self.glow:Hide()
    elseif (self.owner.id == 'player') and( IsResting() ) then
		self.glow:SetVertexColor(1.0, 0.88, 0.25, 1.0)
		self.glow:Show()
    else
		self.glow:SetVertexColor(1,1,1,1)
		self.glow:Hide()
    end
	if ( self.glow:IsShown() ) then
		self.statusCounter = self.statusCounter or 0
		local alpha = 255
		local counter = self.statusCounter + elapsed
		local sign    = self.statusSign
		if ( counter > 0.5 ) then
		    sign = -sign
		    self.statusSign = sign
		end
		counter = mod(counter, 0.5)
		self.statusCounter = counter
 
		if ( sign == 1 ) then
		    alpha = (55  + (counter * 400)) / 255
		else
		    alpha = (255 - (counter * 400)) / 255
		end
		self.glow:SetAlpha(alpha)
    end

	local classification = UnitClassification(self.owner.id)
	if self.last_classification ~= classification then
	    self.last_classification = classification
	
        local texture
        if ( classification == 'minus' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame-Minus')
            --self.power:Hide()
        elseif ( classification == 'worldboss' or classification == 'elite' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame-Elite')
        elseif ( classification == 'rareelite' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame-Rare-Elite')
        elseif ( classification == 'rare' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame-Rare')
        else
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame')
        end
        self.texture:SetTexture(texture)

		
		if self.sets.flipL2R == true then
			self.texture:SetTexCoord(1, 0.09375, 0, 0.78125)
		else
			self.texture:SetTexCoord(0.09375, 1.0, 0, 0.78125)
		end
		
		
    end
	if self.sets.enable == true then
		self:SetAlpha(1)
	else
		self:SetAlpha(0)
	end
end
widget.Options = {
	
                    {
                        name = 'Flip',
                        kind = 'CheckButton',
                        key = 'flipL2R',
						panel = "basic",
                    },
}
