local Addon = _G[...]
local modName = ...

local hori = {'LEFT', 'CENTER', 'RIGHT'}
local vert = {'TOP',  'MIDDLE', 'BOTTOM'}

local function FormatValue(value)
	if (value <= 1000) then
		return value
	elseif (value < 1000000) then
		return ('%.1fk'):format(value / 1000);
	elseif (value < 1000000000) then
		return ('%.2fm'):format(value / 1000000);
	else
		return ('%.2fg'):format(value / 1000000000);
	end
end

local function FormatBarValues(value,max,type)
	if (type == 'none') or (value == nil) then
		return ''
	elseif (type == 'value') or (max == 0) then
		return string.format('%s / %s',AbbreviateLargeNumbers(value),AbbreviateLargeNumbers(max))
	elseif (type == 'current') then
		return string.format('%s',AbbreviateLargeNumbers(value))
	elseif (type == 'full') then
		return string.format('%s / %s (%.0f%%)',FormatValue(value),FormatValue(max),value / max * 100)
	elseif (type == 'deficit') then
		if (value ~= max) then
			return string.format('-%s',FormatValue(max - value))
		else
			return ''
		end
	elseif (type == 'percent') then
		return string.format('%.0f%%',value / max * 100)
	end
end

local statusBar = {}

function statusBar:New(parent, title)
	local name = ('%s_%s_'..title):format(modName, parent.id)
	if _G[name] then return _G[name] end

	local bar = CreateFrame("StatusBar", name, parent.box, 'TextStatusBar')
	bar:SetStatusBarTexture('Interface\\RaidFrame\\Raid-Bar-Hp-Fill', 'BORDER')
	bar:SetStatusBarColor(0,1,0,1)
	bar:EnableMouse(false)
	bar:Show()
	bar:SetScript('OnEnter', nil)

	bar.text = bar.text or bar:CreateFontString(nil, 'ARTWORK', 'TextStatusBarText')
	bar.text:SetTextColor(1.0,1.0,1.0)
	--bar.text:SetAllPoints(true)

	bar.drop = bar.drop or CreateFrame('StatusBar', nil, bar)
	bar.drop:SetMinMaxValues(0,1)
	bar.drop:SetValue(1)
	bar.drop:SetAllPoints(bar)
	bar.drop:SetFrameLevel(bar:GetFrameLevel()-2)

	bar.testTexture = 	bar.testTexture or CreateFrame('StatusBar', nil)
	bar.testTexture:SetMinMaxValues(0,1)
	bar.testTexture:SetValue(1)
	bar.testTexture:SetAllPoints(bar)
	bar.testTexture:SetFrameLevel(bar:GetFrameLevel()-2)
	
	bar.owner = parent
	bar.title = title
	
	Addon.Merge(CopyTable(statusBar), bar)
	
	bar.handler = parent.id
	
	return bar
end

statusBar.defaults = {
	scale = 100,
	height = 12,
	justifyV = 2,
	width = 51,
	x = -106,
	y = -41,
	justifyH = 2,
	drop = true,
	anchor = 'TopRight',
	enable = true,
	percent = true,
	alpha = 100,
	format = 'value',
	mouseover = 'value',
	textureName = 'Raid',
	texturePath = 'Interface\\RaidFrame\\Raid-Bar-Hp-Fill',
	textHori = 2,
	textVert = 2,
	orient = "HORIZONTAL",
	prevOrient = "HORIZONTAL",
    mouseOverTooltip = true,
	arc = 0,
	textX = 0,
	textY = 0,
	maxTextWidth = 98,
	reverseArc = nil,
    CurveOffset = 0,
    topOffset = 0,
    bottomOffset = 100,
    leftOffset = 0,
    rightOffset = 100,
    backdropOpacity = 50,
	LifeTime = 0,
	CurrentPlayTime = 0,
	frameLevel = 2,
	frameLayer = 4,
	anchorText = "Center",
}

function statusBar:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	self:SetPoint(sets.anchor, self:GetParent():GetParent(), sets.x, sets.y)
	self.text:ClearAllPoints()
	self.text:SetPoint(sets.anchorText, self, sets.textX, sets.textY)
	
	local width, height = (sets.width), (sets.height)
	self:SetSize(width, height)
	if sets.scale < 25 then
		sets.scale = sets.scale *100
	end
	self:SetScale(sets.scale/100)
	if self.opac then
		self.opac:SetAlpha(sets.alpha/100)
	end
	local anchor = sets.anchor
	
	if type(sets.justifyH) ~= "number" then
		sets.justifyH = 2
	end
	if type(sets.justifyV) ~= "number" then
		sets.justifyV = 2
	end	
	if self.text then 
		self.text:SetJustifyH(hori[sets.justifyH] or sets.justifyH or "CENTER") -- Sets horizontal text justification ('LEFT','RIGHT', or 'CENTER')
		self.text:SetJustifyV(vert[sets.justifyV] or sets.justifyV or 'MIDDLE') -- Sets vertical text justification ('TOP','BOTTOM', or 'MIDDLE')
		self.text:SetWidth(sets.maxTextWidth)
	end

    if sets.drop then
        self.drop:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
        self.drop:SetStatusBarColor(0,0,0,self.sets.backdropOpacity/100)
    else
        self.drop:SetStatusBarTexture("")
    end
	
	
	if sets.testTexture then
        self.testTexture:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
        self.testTexture:SetStatusBarColor(1,0,0,self.sets.backdropOpacity/100)
    else
        self.testTexture:SetStatusBarTexture("")
    end
	
	
	self:SetOrientation(sets.orient) -- "HORIZONTAL" or "VERTICAL"
	self.drop:SetOrientation(sets.orient)
	self.testTexture:SetOrientation(sets.orient)
	
    if (self.prevTexture ~= self:GetMediaPath('statusbar', self.sets.textureName)) or
    (self.prevTexture ~= self.sets.texturePath) then
	    self:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
    end
    self:SetRotatesTexture(self.sets.rotate)
    self.drop:SetRotatesTexture(self.sets.rotate)
    self.testTexture:SetRotatesTexture(self.sets.rotate)
	
    if self.SetSpark then self:SetSpark() end
	self:Update()
end

function statusBar:Update()
	if self.noUpdate then
		return
	end
	if self.OnUpdate then
		self:OnUpdate()
	end
end

function statusBar:SetValues(current, minimum, maximum, overide, overideText)
	self:SetMinMaxValues(minimum or 0, maximum or 0)
	if ((overide) or (maximum <= 0)or (maximum == nil)) and self.SetText then
		self:SetValue(math.max(minimum, 0))
		self:SetText(overideText)
		--self.text:Hide()
		return	
	end
	local sets = self.sets
	if self.sets.mouseover ~= 'none' and (MouseIsOver(self)) then
		self.text:SetText(FormatBarValues(current, maximum, self.sets.mouseover))
	else
		self.text:SetText(FormatBarValues(current, maximum, self.sets.format))
	end
	--self.text:Hide()
	self:SetValue(current or 0)
end

function statusBar:SetColor(r, g, b, a)
	if ( r ~= self.r or g ~= self.g or b ~= self.b) or (a ~= self.a) then
		self:SetStatusBarColor(r, g, b)
		self.r, self.g, self.b, self.a = r, g, b, a
	end
end

function statusBar:Rescale()
	self:SetScale(self.sets.scale/100)
end

function statusBar:Reposition()
	self:ClearAllPoints()
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	self:SetPoint(self.sets.anchor, self:GetParent(), x, y)
end

function statusBar:GetMediaPath(kind, fileName)
	if Addon.lib then
		self.sets.texturePath = Addon.lib and Addon.lib:Fetch(kind, fileName)
	end
	return (Addon.lib and Addon.lib:Fetch(kind, fileName))
end

local template = Addon:NewTemplate("StatusBar")
template.CreateNew  = statusBar.New

function template:Load()
end

template.Options = {

					{
						name = 'Texture',
						kind = 'Media',
						key = 'textureName',
						mediaType = 'StatusBar',
						panel = "texture",
					},
                    {
                        name = 'Backdrop',
                        kind = 'CheckButton',
                        key = 'drop',
						panel = "texture",
                    },
                    {
                        name = 'Rotate 90',
                        kind = 'CheckButton',
                        key = 'rotate',
						panel = "texture",
                    },
                    {
                        name = 'Growth',
                        kind = 'Menu',
                        key = 'orient',
                        table = {"HORIZONTAL", "VERTICAL"},
						panel = "texture",
                    },

                    {
                        name = 'Background Alpha',
                        kind = 'Slider',
                        key = 'backdropOpacity',
                        min = 0,
                        max = 100,
						panel = "texture",
                    },	

                    {
                        name = 'Format',
                        kind = 'Menu',
                        key = 'format',
                        table = {
                            'none',
                            'value',
                            'current',
                            'full',
                            'deficit',
                            'percent',
                        },
						panel = "text",
                    },
                    {
                        name = 'MouseOver Format',
                        kind = 'Menu',
                        key = 'mouseover',
                        table = {
                            'none',
                            'value',
                            'current',
                            'full',
                            'deficit',
                            'percent',
                        },
						panel = "text",
                    },
                    {
                        name = 'Anchor',
                        kind = 'Menu',
                        key = 'anchorText',
                        table = Addon.anchors,
						panel = "text",
                    },
                    {
                        name = 'Max Width',
                        kind = 'Slider',
                        key = 'maxTextWidth',
                        min = 24,
                        max = 300,
						panel = "text",
                    },
                    {
                        name = 'Y offset',
                        kind = 'Slider',
                        key = 'textY',
                        min = -150,
                        max = 150,
						panel = "text",
                    },	
                    {
                        name = 'X offset',
                        kind = 'Slider',
                        key = 'textX',
                        min = -150,
                        max = 150,
						panel = "text",
                    },

					
					
                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 5,
                        max = 250,
						panel = "basic",
                    },
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 5,
                        max = 250,
						panel = "basic",
                    },

                    {
                        name = 'Tooltip',
                        kind = 'CheckButton',
                        key = 'mouseOverTooltip',
						panel = "advanced",
                    },
                    {
                        name = 'Justfy Veritcal',
                        kind = 'Slider',
                        key = 'justifyV',
                        min = 1,
                        max = 3,
						panel = "text",
                    },	
                    {
                        name = 'Justfy Horizontal',
                        kind = 'Slider',
                        key = 'justifyH',
                        min = 1,
                        max = 3,
						panel = "text",
                    },	
					
        }
