local Addon = _G[...]
local modName = ...

--Threat Text
local widget = Addon:NewWidget('threat', 'Frame')
widget.Events = {
	["UNIT_THREAT_LIST_UPDATE"] = 1,
	["UNIT_THREAT_SITUATION_UPDATE"] = 1,
}

function widget:New(parent)
	local name = ('%s_%s_ThreatText'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local control = self:Bind(CreateFrame('Frame', name, parent.box))
	control.owner = parent
	control:SetFrameLevel(40)
	local string = control:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	string:SetAllPoints(control)
	control.text = string
    control.noMouse = true
	return control
end

widget.defaults = {
	scale = 100,
	justifyV = 1,
	width = 35,
	y = 12,
	x = 0,
	justifyH = 3,
	height = 12,
	enable = true,
	anchor = 'top',
}

local _hori = {
	'LEFT',
	'CENTER',
	'RIGHT',
}

local _vert = {
	'TOP',
	'MIDDLE',
	'BOTTOM',
}

function widget:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.text:SetText('')
		return
	else
		self:Show()
	end
	self:SetScale(self.sets.scale/100)
	self:SetSize(sets.width, sets.height)
	self.text:SetJustifyH(_hori[sets.justifyH]) -- Sets horizontal text justification ('LEFT','RIGHT', or 'CENTER')
	self.text:SetJustifyV(_vert[sets.justifyV]) -- Sets vertical text justification ('TOP','BOTTOM', or 'MIDDLE')
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	self:ClearAllPoints()
	self:SetPoint(self.sets.anchor, self:GetParent(), x, y)
end

function widget:Update()
	if self.sets.enable ~= true then
		return
	end
    local feedbackUnit, unit  = 'player', self.owner.id
	if (UnitClassification(unit) ~= "minus") and
     ( ShowNumericThreat() and not (UnitClassification(self.owner) == "minus") ) and
	 (feedbackUnit and unit) and (UnitName(feedbackUnit) ~= UnitName(unit)) and  (not UnitIsDead(unit)) and (UnitExists(unit)) then
        local isTanking, status, percentage, rawPercentage = UnitDetailedThreatSituation(feedbackUnit, unit);
        local display = rawPercentage;
        if ( isTanking ) then
            display = UnitThreatPercentageOfLead(feedbackUnit, unit);
        end
		if ( display and display ~= 0 ) then
			self.text:SetText(format("%1.0f", display).."%")
		else
			self.text:SetText('')
		end
	else
		self.text:SetText('')
	end
	if self.testMode then
		self.text:SetText('200%')

	end
end

function widget:ToggleTestMode()
	self.testMode = not self.testMode

	self:Update()
end

widget.Options = {
					{
						kind = "Button",
						name = "Test Mode",
						handler = "threat",
						func = function(owner)
							owner:ToggleTestMode()
						end,
						panel = "basic",
					},
                    {
                        name = 'Justify Vert',
                        kind = 'Slider',
                        key = 'justifyV',
                        min = 1,
                        max = 3,
						panel = "text",
                    },
                    {
                        name = 'Justify Hori',
                        kind = 'Slider',
                        key = 'justifyH',
                        min = 1,
                        max = 3,
						panel = "text",
                    },


                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 10,
                        max = 400,
                        step = 1,
						panel = "text",
                    },
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 12,
                        max = 240,
                        step = 14,
						panel = "text",
                    },

        }


